@@ -30,15 +30,8 @@ import { pointWithin, Quadtree, Rect } from '../barchart/quadtree';
 
 import { DEFAULT_POINT_SIZE } from './config';
 import { isGraphable } from './dims';
-import {
-  DimensionValues,
-  ScatterFieldConfig,
-  defaultScatterFieldConfig,
-  ScatterHoverCallback,
-  ScatterSeries,
-  Options,
-  ScatterShow,
-} from './types';
+import { FieldConfig, defaultFieldConfig, Options, ScatterShow } from './panelcfg.gen';
+import { DimensionValues, ScatterHoverCallback, ScatterSeries } from './types';
 
 export interface ScatterPanelInfo {
   error?: string;
@@ -53,9 +46,9 @@ export function prepScatter(
   options: Options,
   getData: () => DataFrame[],
   theme: GrafanaTheme2,
-  ttip: ScatterHoverCallback,
+  ttip: null | ScatterHoverCallback,
   onUPlotClick: null | ((evt?: Object) => void),
-  isToolTipOpen: MutableRefObject<boolean>
+  isToolTipOpen: null | MutableRefObject<boolean>
 ): ScatterPanelInfo {
   let series: ScatterSeries[];
   let builder: UPlotConfigBuilder;
@@ -111,7 +104,7 @@ function getScatterSeries(
     ? config.theme2.visualization.getColorByName(dims.pointColorFixed)
     : getFieldSeriesColor(y, config.theme2).color;
   let pointColor: DimensionValues<string> = () => seriesColor;
-  const fieldConfig: ScatterFieldConfig = { ...defaultScatterFieldConfig, ...y.config.custom };
+  const fieldConfig: FieldConfig = { ...defaultFieldConfig, ...y.config.custom };
   let pointColorMode = fieldColorModeRegistry.get(FieldColorModeId.PaletteClassic);
   if (dims.pointColorIndex) {
     const f = frames[frameIndex].fields[dims.pointColorIndex];
@@ -232,11 +225,11 @@ function prepSeries(options: Options, frames: DataFrame[]): ScatterSeries[] {
 
       for (let frameIndex = 0; frameIndex < frames.length; frameIndex++) {
         const frame = frames[frameIndex];
-        const xIndex = findFieldIndex(frame, series.x);
+        const xIndex = findFieldIndex(series.x, frame, frames);
 
         if (xIndex != null) {
           // TODO: this should find multiple y fields
-          const yIndex = findFieldIndex(frame, series.y);
+          const yIndex = findFieldIndex(series.y, frame, frames);
 
           if (yIndex == null) {
             throw 'Y must be in the same frame as X';
@@ -244,9 +237,9 @@ function prepSeries(options: Options, frames: DataFrame[]): ScatterSeries[] {
 
           const dims: Dims = {
             pointColorFixed: series.pointColor?.fixed,
-            pointColorIndex: findFieldIndex(frame, series.pointColor?.field),
+            pointColorIndex: findFieldIndex(series.pointColor?.field, frame, frames),
             pointSizeConfig: series.pointSize,
-            pointSizeIndex: findFieldIndex(frame, series.pointSize?.field),
+            pointSizeIndex: findFieldIndex(series.pointSize?.field, frame, frames),
           };
           scatterSeries.push(getScatterSeries(seriesIndex++, frames, frameIndex, xIndex, yIndex, dims));
         }
@@ -262,7 +255,7 @@ function prepSeries(options: Options, frames: DataFrame[]): ScatterSeries[] {
   const frame = frames[frameIndex];
   const numericIndices: number[] = [];
 
-  let xIndex = findFieldIndex(frame, dims.x);
+  let xIndex = findFieldIndex(dims.x, frame, frames);
   for (let i = 0; i < frame.fields.length; i++) {
     if (isGraphable(frame.fields[i])) {
       if (xIndex == null || i === xIndex) {
@@ -301,14 +294,13 @@ interface DrawBubblesOpts {
   };
 }
 
-//const prepConfig: UPlotConfigPrepFnXY<XYChartOptions> = ({ frames, series, theme }) => {
 const prepConfig = (
   getData: () => DataFrame[],
   scatterSeries: ScatterSeries[],
   theme: GrafanaTheme2,
-  ttip: ScatterHoverCallback,
+  ttip: null | ScatterHoverCallback,
   onUPlotClick: null | ((evt?: Object) => void),
-  isToolTipOpen: MutableRefObject<boolean>
+  isToolTipOpen: null | MutableRefObject<boolean>
 ) => {
   let qt: Quadtree;
   let hRect: Rect | null;
@@ -400,6 +392,9 @@ const prepConfig = (
               }
 
               if (showPoints) {
+                // if pointHints.fixed? don't recalc size
+                // if pointColor has 0 opacity, draw as single path (assuming all strokes are alpha 1)
+
                 u.ctx.moveTo(cx + size / 2, cy);
                 u.ctx.beginPath();
                 u.ctx.arc(cx, cy, size / 2, 0, deg360);
@@ -526,8 +521,10 @@ const prepConfig = (
   });
 
   const clearPopupIfOpened = () => {
-    if (isToolTipOpen.current) {
-      ttip(undefined);
+    if (isToolTipOpen?.current) {
+      if (ttip) {
+        ttip(undefined);
+      }
       if (onUPlotClick) {
         onUPlotClick();
       }
@@ -538,7 +535,9 @@ const prepConfig = (
 
   // clip hover points/bubbles to plotting area
   builder.addHook('init', (u, r) => {
-    u.over.style.overflow = 'hidden';
+    if (!config.featureToggles.newVizTooltips) {
+      u.over.style.overflow = 'hidden';
+    }
     ref_parent = u.root.parentElement;
 
     if (onUPlotClick) {
@@ -560,26 +559,28 @@ const prepConfig = (
     rect = r;
   });
 
-  builder.addHook('setLegend', (u) => {
-    if (u.cursor.idxs != null) {
-      for (let i = 0; i < u.cursor.idxs.length; i++) {
-        const sel = u.cursor.idxs[i];
-        if (sel != null && !isToolTipOpen.current) {
-          ttip({
-            scatterIndex: i - 1,
-            xIndex: sel,
-            pageX: rect.left + u.cursor.left!,
-            pageY: rect.top + u.cursor.top!,
-          });
-          return; // only show the first one
+  if (ttip) {
+    builder.addHook('setLegend', (u) => {
+      if (u.cursor.idxs != null) {
+        for (let i = 0; i < u.cursor.idxs.length; i++) {
+          const sel = u.cursor.idxs[i];
+          if (sel != null && !isToolTipOpen?.current) {
+            ttip({
+              scatterIndex: i - 1,
+              xIndex: sel,
+              pageX: rect.left + u.cursor.left!,
+              pageY: rect.top + u.cursor.top!,
+            });
+            return; // only show the first one
+          }
         }
       }
-    }
 
-    if (!isToolTipOpen.current) {
-      ttip(undefined);
-    }
-  });
+      if (!isToolTipOpen?.current) {
+        ttip(undefined);
+      }
+    });
+  }
 
   builder.addHook('drawClear', (u) => {
     clearPopupIfOpened();
@@ -602,8 +603,8 @@ const prepConfig = (
   const frames = getData();
   let xField = scatterSeries[0].x(scatterSeries[0].frame(frames));
 
-  let config = xField.config;
-  let customConfig = config.custom;
+  let fieldConfig = xField.config;
+  let customConfig = fieldConfig.custom;
   let scaleDistr = customConfig?.scaleDistribution;
 
   builder.addScale({
@@ -614,12 +615,12 @@ const prepConfig = (
     distribution: scaleDistr?.type,
     log: scaleDistr?.log,
     linearThreshold: scaleDistr?.linearThreshold,
-    min: config.min,
-    max: config.max,
+    min: fieldConfig.min,
+    max: fieldConfig.max,
     softMin: customConfig?.axisSoftMin,
     softMax: customConfig?.axisSoftMax,
     centeredZero: customConfig?.axisCenteredZero,
-    decimals: config.decimals,
+    decimals: fieldConfig.decimals,
   });
 
   // why does this fall back to '' instead of null or undef?
@@ -629,6 +630,8 @@ const prepConfig = (
     scaleKey: 'x',
     placement: customConfig?.axisPlacement !== AxisPlacement.Hidden ? AxisPlacement.Bottom : AxisPlacement.Hidden,
     show: customConfig?.axisPlacement !== AxisPlacement.Hidden,
+    grid: { show: customConfig?.axisGridShow },
+    border: { show: customConfig?.axisBorderShow },
     theme,
     label:
       xAxisLabel == null || xAxisLabel === ''
@@ -666,21 +669,23 @@ const prepConfig = (
       decimals: config.decimals,
     });
 
-    if (customConfig?.axisPlacement !== AxisPlacement.Hidden) {
-      // why does this fall back to '' instead of null or undef?
-      let yAxisLabel = customConfig?.axisLabel;
+    // why does this fall back to '' instead of null or undef?
+    let yAxisLabel = customConfig?.axisLabel;
 
-      builder.addAxis({
-        scaleKey,
-        theme,
-        placement: customConfig?.axisPlacement,
-        label:
-          yAxisLabel == null || yAxisLabel === ''
-            ? getFieldDisplayName(field, scatterSeries[si].frame(frames), frames)
-            : yAxisLabel,
-        formatValue: (v, decimals) => formattedValueToString(field.display!(v, decimals)),
-      });
-    }
+    builder.addAxis({
+      scaleKey,
+      theme,
+      placement: customConfig?.axisPlacement === AxisPlacement.Auto ? AxisPlacement.Left : customConfig?.axisPlacement,
+      show: customConfig?.axisPlacement !== AxisPlacement.Hidden,
+      grid: { show: customConfig?.axisGridShow },
+      border: { show: customConfig?.axisBorderShow },
+      size: customConfig?.axisWidth,
+      label:
+        yAxisLabel == null || yAxisLabel === ''
+          ? getFieldDisplayName(field, scatterSeries[si].frame(frames), frames)
+          : yAxisLabel,
+      formatValue: (v, decimals) => formattedValueToString(field.display!(v, decimals)),
+    });
 
     builder.addSeries({
       facets: [
@@ -696,7 +701,7 @@ const prepConfig = (
       pathBuilder: drawBubbles, // drawBubbles({disp: {size: {values: () => }}})
       theme,
       scaleKey: '', // facets' scales used (above)
-      lineColor: lineColor as string,
+      lineColor: alpha('' + lineColor, 1),
       fillColor: alpha(pointColor, 0.5),
       show: !customConfig.hideFrom?.viz,
     });
